package com.dsk.cscec.controller;

import cn.hutool.core.lang.Assert;
import com.dsk.common.core.controller.BaseController;
import com.dsk.common.core.domain.PageQuery;
import com.dsk.common.core.domain.R;
import com.dsk.common.core.page.TableDataInfo;
import com.dsk.cscec.domain.CbProjectRecord;
import com.dsk.cscec.domain.bo.AddCbProjectBo;
import com.dsk.cscec.domain.bo.AddNewCbProjectBo;
import com.dsk.cscec.domain.bo.CbProjectSearchBo;
import com.dsk.cscec.domain.bo.EditProjectInfoBo;
import com.dsk.cscec.domain.vo.ProjectRecordSearchVo;
import com.dsk.cscec.service.CbProjectRecordService;
import com.dsk.cscec.service.IDProjectService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 项目台账表(CbProjectRecord)表控制层
 *
 * @author sxk
 * @since 2024-02-05 11:37:49
 */
@RestController
@RequestMapping("cbProjectRecord")
public class CbProjectRecordController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private CbProjectRecordService baseService;
    @Resource
    private IDProjectService dProjectService;

    /**
     * 校验项目编码是否存在
     */
    @GetMapping("/checkProjectCodeExist/{projectCode}")
    public R<Boolean> checkProjectCodeExist(@NotBlank(message = "项目编码不能为空") @PathVariable String projectCode) {
        return R.ok(dProjectService.checkProjectCodeExist(projectCode));
    }

    /**
     * 新增项目
     */
    @PostMapping("/addProject")
    public R<CbProjectRecord> addProject(@Validated @RequestBody AddCbProjectBo addCbProjectBo) {
        CbProjectRecord projectRecord = baseService.addProject(addCbProjectBo);
        Assert.notNull(projectRecord, "新增项目失败,请联系管理员");
        return R.ok(projectRecord);
    }

    /**
     * 根据项目文件导入数据
     */
    @PostMapping("/importData/{projectId}")
    public R<Void> importData(@NotNull(message = "项目ID不能为空") @PathVariable Long projectId) throws Exception {
        baseService.importData(projectId);
        return R.ok("已开始解析成本数据,过程需3~10分钟,稍后可到项目列表查看导入结果");
    }

    /**
     * 获取项目台账列表
     */
    @GetMapping("/getProjectList")
    public TableDataInfo<ProjectRecordSearchVo> getProjectList(CbProjectSearchBo searchBo, PageQuery pageQuery) {
        return baseService.getProjectList(searchBo, pageQuery);
    }

    /**
     * 修改项目信息
     */
    @PutMapping("/editProjectInfo")
    public R<Void> editProjectInfo(@Validated @RequestBody EditProjectInfoBo editBo) {
        return toAjax(baseService.editProjectInfo(editBo));
    }

    /**
     * 获取新阶段项目可选成本阶段
     */
    @GetMapping("/getProjectAvailableCbStage/{relatedId}")
    public R<List<Integer>> getProjectAvailableCbStage(@NotNull(message = "关联ID不能为空") @PathVariable Long relatedId) {
        return R.ok(baseService.getProjectAvailableCbStage(relatedId));
    }

    /**
     * 新增新阶段项目
     */
    @PostMapping("/addNewStageProject")
    public R<CbProjectRecord> addNewStageProject(@Validated @RequestBody AddNewCbProjectBo newProjectBo) {
        return R.ok(baseService.addNewStageProject(newProjectBo));
    }

    /**
     * 获取草稿弹窗列表
     */
    @GetMapping("/getDraftDialogList")
    public R<List<CbProjectRecord>> getDraftDialogList() {
        List<CbProjectRecord> draftList = baseService.getDraftDialogList();
        if (draftList.isEmpty()) {
            return R.ok("暂无草稿记录");
        }
        return R.ok(draftList);
    }

    /**
     * 删除草稿
     */
    @DeleteMapping("/deleteDraft/{projectId}")
    public R<Void> deleteDraft(@NotNull(message = "项目ID不能为空") @PathVariable Long projectId) {
        return toAjax(baseService.deleteDraft(projectId));
    }
}

