package com.dsk.cscec.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsk.common.core.domain.PageQuery;
import com.dsk.common.core.page.TableDataInfo;
import com.dsk.common.utils.StringUtils;
import com.dsk.component.DataAnalysisComponent;
import com.dsk.cscec.constant.CbProjectConstants;
import com.dsk.cscec.domain.CbProjectFile;
import com.dsk.cscec.domain.CbProjectRecord;
import com.dsk.cscec.domain.bo.*;
import com.dsk.cscec.domain.vo.ProjectRecordSearchVo;
import com.dsk.cscec.mapper.CbProjectFileMapper;
import com.dsk.cscec.mapper.CbProjectRecordMapper;
import com.dsk.cscec.service.CbProjectRecordService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 项目台账表(CbProjectRecord)表服务实现类
 *
 * @author sxk
 * @since 2024-02-05 11:37:53
 */
@Service("cbProjectRecordService")
public class CbProjectRecordServiceImpl extends ServiceImpl<CbProjectRecordMapper, CbProjectRecord> implements CbProjectRecordService {
    @Resource
    private CbProjectRecordMapper baseMapper;
    @Resource
    private CbProjectFileMapper projectFileMapper;
    @Resource
    private DataAnalysisComponent dataAnalysisComponent;

    /**
     * 新增项目
     *
     * @param addCbProjectBo 新增对象
     * @return 新增结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public CbProjectRecord addProject(AddCbProjectBo addCbProjectBo) {
        CbProjectRecord projectRecord = BeanUtil.toBean(addCbProjectBo, CbProjectRecord.class);

        //生成唯一关联ID
        boolean idFlag = true;
        long relatedId = -1;
        while (idFlag) {
            relatedId = IdUtil.getSnowflakeNextId();
            idFlag = baseMapper.exists(new LambdaQueryWrapper<CbProjectRecord>()
                    .eq(CbProjectRecord::getRelatedId, relatedId));
        }

        projectRecord.setRelatedId(relatedId);
        //项目文件状态:准备中
        projectRecord.setProjectFileStatus(CbProjectConstants.PROJECT_FILE_STATUS_PREPARING);
        //删除状态:否
        projectRecord.setDelFlag(CbProjectConstants.DELETE_FLAG_EXIST);
        if (baseMapper.insert(projectRecord) > 0) {
            return projectRecord;
        }
        return null;
    }

    /**
     * 根据项目文件导入数据
     *
     * @param projectId 项目ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void importData(Long projectId) throws Exception {
        //校验项目是否存在
        CbProjectRecord projectRecord = this.checkProjectExist(projectId);

        //校验各个成本类型是否至少有1个文件
        //直接费成本
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_DIRECT_EXPENSE, CbProjectConstants.CB_TYPE_DIRECT_EXPENSE_NAME);
        //工料汇总
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_QUANTITY_SUMMARY, CbProjectConstants.CB_TYPE_QUANTITY_SUMMARY_NAME);
        //措施项目
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_MEASURE_PROJECT, CbProjectConstants.CB_TYPE_MEASURE_PROJECT_NAME);
        //其他项目
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_OTHER_PROJECT, CbProjectConstants.CB_TYPE_OTHER_PROJECT_NAME);
        //现场经费
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_SCENE_EXPENSE, CbProjectConstants.CB_TYPE_SCENE_EXPENSE_NAME);
        //成本汇总
        this.checkCbTypeExistFile(projectId, CbProjectConstants.CB_TYPE_SUMMARY, CbProjectConstants.CB_TYPE_SUMMARY_NAME);

        //修改项目文件状态:解析中
        projectRecord.setProjectFileStatus(CbProjectConstants.PROJECT_FILE_STATUS_PARSING);

        //TODO:调各个成本类型的解析文件方法
        //直接费成本
        //工料汇总
        CbProjectBaseBo projectBaseBo = new CbProjectBaseBo();
        projectBaseBo.setProjectId(projectId);
        projectBaseBo.setCbStage(projectRecord.getCbStage());
        dataAnalysisComponent.quantitySummaryDataAnalysis(projectBaseBo);
        //措施项目
        //其他项目
        //现场经费
        //成本汇总

    }

    /**
     * 校验各个成本类型是否至少有1个文件
     *
     * @param projectId  项目ID
     * @param cbType     成本类型字典（0:直接费成本、1:工料汇总、2:措施项目、3:其他项目、4:现场经费、5:成本汇总）
     * @param cbTypeName 成本类型名称
     */
    private void checkCbTypeExistFile(Long projectId, Integer cbType, String cbTypeName) {
        boolean isExist = projectFileMapper.exists(new LambdaQueryWrapper<CbProjectFile>()
                .eq(CbProjectFile::getProjectId, projectId)
                .eq(CbProjectFile::getCbType, cbType)
                //项目文件状态:解析中
                .eq(CbProjectFile::getFileParseStatus, CbProjectConstants.PROJECT_FILE_STATUS_PREPARING));
        Assert.isTrue(isExist, cbTypeName + "至少需要上传1个文件");
    }

    /**
     * 获取项目台账列表
     *
     * @param searchBo  查询条件
     * @param pageQuery 分页对象
     * @return 分页数据
     */
    @Override
    public TableDataInfo<ProjectRecordSearchVo> getProjectList(CbProjectSearchBo searchBo, PageQuery pageQuery) {
        QueryWrapper<CbProjectRecord> wrapper = Wrappers.query();
        wrapper
                .like(StringUtils.isNotBlank(searchBo.getProjectName()), "project_name", searchBo.getProjectName())
                .like(StringUtils.isNotBlank(searchBo.getIpmProjectNo()), "ipm_project_no", searchBo.getIpmProjectNo());

        //TODO:成本阶段、文件解析状态筛选
        Integer cbStage = searchBo.getCbStage();
        Integer projectFileStatus = searchBo.getProjectFileStatus();

        Page<ProjectRecordSearchVo> page = baseMapper.selectPageProjectRecordList(pageQuery.build(), wrapper);
        for (ProjectRecordSearchVo searchVo : page.getRecords()) {
            Long projectId = searchVo.getId();
            //page.
        }
        return TableDataInfo.build(page);
    }

    /**
     * 修改项目信息
     *
     * @param editBo 修改对象
     * @return 修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer editProjectInfo(EditProjectInfoBo editBo) {
        //校验项目是否存在
        CbProjectRecord projectRecord = this.checkProjectExist(editBo.getProjectId());

        for (CbProjectRecord record : baseMapper.selectList(new LambdaQueryWrapper<CbProjectRecord>()
                .eq(CbProjectRecord::getRelatedId, projectRecord.getRelatedId()))) {
            record.setProjectName(editBo.getProjectName());
            record.setIpmProjectNo(editBo.getIpmProjectNo());
            record.setIsGetProjectDetail(editBo.getIsGetProjectDetail());
            record.setProjectFileName(editBo.getProjectFileName());
            Assert.isTrue(baseMapper.updateById(record) > 0, "修改失败,请联系系统管理员");
        }
        return 1;
    }

    /**
     * 获取新阶段项目可选成本阶段
     *
     * @param relatedId 关联ID
     * @return 可选成本阶段
     */
    @Override
    public List<Integer> getProjectAvailableCbStage(Long relatedId) {
        //通过关联ID校验项目是否存在
        List<CbProjectRecord> projectRecordList = this.checkRelatedIdExist(relatedId);

        ArrayList<Integer> cbStageList = new ArrayList<>();
        cbStageList.add(CbProjectConstants.CB_STAGE_BEFORE_BID);
        cbStageList.add(CbProjectConstants.CB_STAGE_AFTER_BID);
        cbStageList.add(CbProjectConstants.CB_STAGE_TO_SOLID);

        projectRecordList.forEach(projectRecord -> {
            cbStageList.remove(projectRecord.getCbStage());
        });
        return cbStageList;
    }

    /**
     * 新增新阶段项目
     *
     * @param newProjectBo 新增对象
     * @return 新增结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public CbProjectRecord addNewStageProject(AddNewCbProjectBo newProjectBo) {
        //修改基础信息
        this.editProjectInfo(BeanUtil.toBean(newProjectBo, EditProjectInfoBo.class));

        //通过关联ID校验项目是否存在
        List<CbProjectRecord> projectRecordList = this.checkRelatedIdExist(newProjectBo.getRelatedId());
        //校验待新增的阶段是否已存在
        projectRecordList.forEach(record -> {
            Assert.isFalse(record.getCbStage().equals(newProjectBo.getCbStage()), "当前项目在该阶段已存在记录,请检查草稿箱或项目台账列表");
        });

        //新增该项目的新阶段
        CbProjectRecord projectRecord = BeanUtil.toBean(newProjectBo, CbProjectRecord.class);
        //项目文件状态:准备中
        projectRecord.setProjectFileStatus(CbProjectConstants.PROJECT_FILE_STATUS_PREPARING);
        //删除状态:否
        projectRecord.setDelFlag(CbProjectConstants.DELETE_FLAG_EXIST);
        if (baseMapper.insert(projectRecord) > 0) {
            return projectRecord;
        }
        return null;
    }

    /**
     * 通过关联ID校验项目是否存在
     *
     * @return 该关联ID下的所有项目记录
     */
    private List<CbProjectRecord> checkRelatedIdExist(Long relatedId) {
        List<CbProjectRecord> projectRecordList = baseMapper.selectList(new LambdaQueryWrapper<CbProjectRecord>().eq(CbProjectRecord::getRelatedId, relatedId));
        Assert.isFalse(projectRecordList.isEmpty(), "关联ID无效");
        return projectRecordList;
    }

    /**
     * 获取草稿弹窗列表
     *
     * @return 草稿弹窗列表
     */
    @Override
    public List<CbProjectRecord> getDraftDialogList() {
        return baseMapper.selectList(new LambdaQueryWrapper<CbProjectRecord>()
                .eq(CbProjectRecord::getProjectFileStatus, CbProjectConstants.PROJECT_FILE_STATUS_PREPARING));
    }

    /**
     * 删除草稿
     *
     * @param projectId 项目ID
     * @return 删除结果
     */
    @Override
    public Integer deleteDraft(Long projectId) {
        //校验项目是否存在
        CbProjectRecord projectRecord = this.checkProjectExist(projectId);
        //删除该项目下已上传的文件
        List<CbProjectFile> idList = projectFileMapper.selectList(new LambdaQueryWrapper<CbProjectFile>()
                .eq(CbProjectFile::getProjectId, projectId));
        if (!idList.isEmpty()) {
            projectFileMapper.deleteBatchIds(idList);
        }
        //删除项目记录
        return baseMapper.deleteById(projectRecord);
    }

    /**
     * 校验项目是否存在
     *
     * @param projectId 项目ID
     * @return 项目实体
     */
    private CbProjectRecord checkProjectExist(Long projectId) {
        CbProjectRecord projectRecord = baseMapper.selectById(projectId);
        Assert.notNull(projectRecord, "该项目不存在");
        return projectRecord;
    }
}
